import os
from sys import platform

import click
import pkg_resources

from agileupstate.terminal import print_check_message, print_cross_message


def get_version_string() -> str:
    return f"AgileUp State Version: {pkg_resources.get_distribution('agileupstate').version}"


def supported_os() -> bool:
    if platform == 'linux' or platform == 'linux2':
        return True
    elif platform == 'darwin':
        return True
    elif platform == 'win32':
        return False


class Client:

    def __init__(self, updated=False):
        self.version = get_version_string()
        self.updated = updated

        if supported_os():
            print_check_message(f'Client machine supported {platform}')
        else:
            print_cross_message(f'Client machine not supported {platform}!', leave=True)

        try:
            self.id = os.environ['SIAB_ID']
        except KeyError:
            print_cross_message('SIAB_ID must be set!', leave=True)
        try:
            self.cloud = os.environ['SIAB_CLOUD']
        except KeyError:
            print_cross_message('SIAB_CLOUD must be set!', leave=True)
        try:
            self.location = os.environ['SIAB_LOCATION']
        except KeyError:
            print_cross_message('SIAB_LOCATION must be set!', leave=True)
        try:
            self.context = os.environ['SIAB_CONTEXT']
        except KeyError:
            print_cross_message('SIAB_CONTEXT must be set!', leave=True)

        try:
            os.environ['VAULT_ADDR']
        except KeyError:
            print_cross_message('VAULT_ADDR must be set!', leave=True)
        try:
            os.environ['VAULT_TOKEN']
        except KeyError:
            print_cross_message('VAULT_TOKEN must be set!', leave=True)

        self.display()

    def display(self) -> None:
        if self.updated:
            message = f'- Updated client ({self.id} {self.cloud} {self.location} {self.context}) {self.version}'
            click.secho(message, fg='yellow')
        else:
            message = f'- Running client ({self.id} {self.cloud} {self.location} {self.context}) {self.version}'
            click.secho(message, fg='blue')

    def validate(self, data: dict) -> None:
        if self.id != data['client-id']:
            print_cross_message(f'Client out of sync error {self.id}!', leave=True)
        if self.cloud != data['client-cloud']:
            print_cross_message(f'Client out of sync error {self.cloud}!', leave=True)
        if self.location != data['client-location']:
            print_cross_message(f'Client out of sync error {self.location}!', leave=True)
        if self.context != data['client-context']:
            print_cross_message(f'Client out of sync error {self.context}!', leave=True)
        self.display()

    def update(self, data: dict) -> None:
        self.id = data['client-id']
        self.cloud = data['client-cloud']
        self.location = data['client-location']
        self.context = data['client-context']
        self.updated = True
        self.display()



import base64
import os

import click
import hvac
from hvac.exceptions import InvalidPath

from agileupstate.state import State
from agileupstate.terminal import print_cross_message, print_check_message


def address():
    return os.environ['VAULT_ADDR']


def is_ready() -> bool:
    client = hvac.Client(
        url=os.environ['VAULT_ADDR'],
        token=os.environ['VAULT_TOKEN']
    )
    if client.sys.is_sealed():
        return print_cross_message(f'Vault is sealed: {address()}', leave=True)
    else:
        if client.is_authenticated():
            print_check_message(f'Vault client authenticated to: {address()}')
            try:
                client.secrets.kv.v2.create_or_update_secret(
                    path='siab/smoke-test',
                    secret=dict(test='this is a vault engine smoke test'),
                )
                list_response = client.secrets.kv.v2.list_secrets(path='siab')
                print_check_message('The following paths are available under "siab" prefix: {keys}'.format(
                    keys=','.join(list_response['data']['keys']),
                ))
                return print_check_message(f'Vault client secrets backend validated: {address()}')
            except InvalidPath:
                return print_cross_message(f'Could not find KV v2 siab path in: {address()}', leave=True)

        else:
            return print_cross_message(f'Vault client failed to authenticate to: {address()}', leave=True)


def create_state() -> State:
    state = State()
    hvac_client = hvac.Client(
        url=os.environ['VAULT_ADDR'],
        token=os.environ['VAULT_TOKEN']
    )
    hvac_client.secrets.kv.v2.create_or_update_secret(
        path=state.vault_state_path,
        secret=state.client_state_data
    )
    click.secho(f'- Created state data in vault {state.vault_state_path}', fg='blue')
    state.write()
    state.write_names()
    return state


def load_state() -> State:
    state = State()
    hvac_client = hvac.Client(
        url=os.environ['VAULT_ADDR'],
        token=os.environ['VAULT_TOKEN']
    )
    try:
        response = hvac_client.secrets.kv.read_secret_version(path=state.vault_state_path)
        click.secho(f'- Loaded state data from vault {state.vault_state_path}', fg='blue')
        click.secho('- Created time: {created_time} Version: {version} '.format(
            created_time=response['data']['metadata']['created_time'],
            version=response['data']['metadata']['version'],
        ), fg='blue')
        state.validate(response['data']['data'])
        state.update(response['data']['data'])
        state.write()
        state.write_names()
        return state
    except InvalidPath:
        print_cross_message(f'Could not find {state.vault_state_path} path in: {address()}', leave=True)


def create_tfstate(state: State, tfstate_content: dict) -> None:
    hvac_client = hvac.Client(
        url=os.environ['VAULT_ADDR'],
        token=os.environ['VAULT_TOKEN']
    )
    hvac_client.secrets.kv.v2.create_or_update_secret(
        path=state.vault_tfstate_path,
        secret=tfstate_content
    )
    click.secho(f'- Created tfstate data in vault {state.vault_tfstate_path}', fg='blue')


def load_tfstate() -> dict:
    state = State()
    hvac_client = hvac.Client(
        url=os.environ['VAULT_ADDR'],
        token=os.environ['VAULT_TOKEN']
    )
    try:
        response = hvac_client.secrets.kv.read_secret_version(path=state.vault_tfstate_path)
        click.secho(f'- Loaded state data from vault {state.vault_tfstate_path}', fg='blue')
        click.secho('- Created time: {created_time} Version: {version} '.format(
            created_time=response['data']['metadata']['created_time'],
            version=response['data']['metadata']['version'],
        ), fg='blue')
        return response['data']['data']
    except InvalidPath:
        print_cross_message(f'Could not find {state.vault_tfstate_path} path in: {address()}', leave=True)


def load_vault_file(path) -> str:
    hvac_client = hvac.Client(
        url=os.environ['VAULT_ADDR'],
        token=os.environ['VAULT_TOKEN']
    )
    try:
        response = hvac_client.secrets.kv.read_secret_version(path=path)
        click.secho(f'- Loaded file data from vault {path}', fg='blue')
        click.secho('- Created time: {created_time} Version: {version} '.format(
            created_time=response['data']['metadata']['created_time'],
            version=response['data']['metadata']['version'],
        ), fg='blue')
        data_base64 = response['data']['data']['file']
        data = base64.b64decode(data_base64)

        dirname, filename = os.path.split(path)
        click.secho(f'- Writing {filename}', fg='blue')
        with open(filename, 'wb') as f:
            f.write(data)

        return filename
    except InvalidPath:
        print_cross_message(f'Could not find {path} path in: {address()}', leave=True)



from zipfile import ZipFile

import click

from agileupstate.ansible import create_inventory, ping_windows
from agileupstate.client import get_version_string
from agileupstate.terminal import print_check_message, print_cross_message
from agileupstate.vault import address, is_ready, create_state, load_state, create_tfstate, load_tfstate, \
    load_vault_file

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.group(help='CLI to manage pipeline states.')
def cli() -> int:
    pass


@cli.command(help='Display the current version.')
def version() -> None:
    click.echo(get_version_string())


@cli.command(help='Check client configuration.')
def check() -> None:
    click.secho('Checking client configuration.', fg='green')
    if is_ready():
        print_check_message('Vault backend is available')
    else:
        print_cross_message(f'Vault backend failed from: {address()}', leave=True)


@cli.command(help='Create client vault states.')
def create() -> None:
    click.secho('- Create client vault states', fg='green')
    create_state()


@cli.command(help='Save client tfstates to vault.')
def save() -> None:
    click.secho('- Save client tfstates to vault', fg='green')
    state = load_state()
    tfstate_content = state.read_tfstate()
    create_tfstate(state, tfstate_content)


@cli.command(help='Load client states from vault.')
def load() -> None:
    click.secho('- Load client states from vault', fg='green')
    state = load_state()
    tfstate_content = load_tfstate()
    state.write_tfstate(tfstate_content)


@cli.command(help='Create ansible inventory from vault tfstate.')
def inventory() -> None:
    click.secho('- Creating ansible inventory from vault tfstate', fg='green')
    state = load_state()
    tfstate_content = state.read_tfstate()
    create_inventory(state, tfstate_content)


@cli.command(help='Generate cloud init zip file for mTLS data.')
@click.option('--server-path', required=True, help='Vault path to windows WinRM server pfx file.')
@click.option('--client-path', required=True, help='Vault path to windows WinRM client pfx file.')
def cloud_init(server_path, client_path) -> None:
    filename1 = load_vault_file(server_path)
    filename2 = load_vault_file(client_path)
    with ZipFile('cloud-init.zip', 'w') as z:
        z.write(filename1)
        z.write(filename2)
    click.secho('- Writing cloud-init.zip', fg='blue')


@cli.command(help='Check connection.')
@click.option('--username', help='Username for windows access.')
@click.option('--password', help='Password for windows access.')
def ping(username, password) -> None:
    click.secho(f'- Checking Windows WinRM connection for user {username}', fg='green')
    ping_windows((username, password))


if __name__ == '__main__':
    exit(cli())
